/*
Gausche Formel
...
Gau stellte folgende Formel zur Berechnung des Osterdatums auf:
Ostern ist am (22+d+e). Mrz oder am(d+e-9). April, je nachdem,
ob (22+d+e) < 32 ist oder nicht. Die Berechnung von d und e ist leider
etwas aufwendig, und es gilt:

j = 100p+n
q = ganzzahliger Teil von p/3
r = ganzzahliger Teil von p/4
x = Dreiigerrest von 15+p-q-r
y = Siebenerrest von p-r+4
a = Neunzehnerrest von j
b = Vierererst von j
c = Siebenerrest von j
d = Dreiigerrest von 19a+x
e = Siebenerrest von 2b+4c+6d+y

Keine Regel ohne Ausnahme:
Ist d = 29 und e = 6, so ist Ostern am 19.April.
Ist d = 28 und e = 6, so ist Ostern am 18.April.

Beispiel fr Ostern 1992:
j = 1992, p = 19, n = 92, q = 6, r = 4, x = 24,
y = 5, a = 16, b = 0, c = 4, d = 28, e = 0,
22+d+e=50 > 32, d+e-9 = 19
Also 19. April

[Quelle: "Weser-Kurier" Bremen vom 18.04.1991]
*/

#include <iostream>

class Easter {
	int j;
	int d() {return (19*a()+x())%30;}
	int e() {return (2*b()+4*c()+6*d()+y())%7;}
	int p() {return j/100;}
	int n() {return j%100;}
	int a() {return j%19;}
	int b() {return j%4;}
	int c() {return j%7;}
	int q() {return p()/3;}
	int r() {return p()/4;}
	int x() {return (15+p()-q()-r())%30;}
	int y() {return (p()-r()+4)%7;}

	static const char* const april=".April";
	static const char* const maerz=".Maerz";
	
public:
	Easter(int year):j(year){}
	void print()
	{
		int i=22+d()+e();
		if (i<32)
			{ cout<<i<<maerz; return;}
		if (d()==29 && e()==6)
			{ cout<<19<<april; return;}
		if (d()==28 && e()==6)
			{ cout<<18<<april; return;}
		cout<<i-22-9<<april;
	}
	void stat()
	{
		cout <<"p="<<p()<<endl;
		cout <<"n="<<n()<<endl;
		cout <<"q="<<q()<<endl;
		cout <<"r="<<r()<<endl;
		cout <<"x="<<x()<<endl;
		cout <<"y="<<y()<<endl;
		cout <<"a="<<a()<<endl;
		cout <<"b="<<b()<<endl;
		cout <<"c="<<c()<<endl;
		cout <<"d="<<d()<<endl;
		cout <<"e="<<e()<<endl;
	}
};

const char* const Easter::april;
const char* const Easter::maerz;

int main()
{
	for (int j=1900; j<2100; ++j)  {
		cout << "Ostern am "; Easter(j).print();
		cout <<" im Jahr "<<j<<endl;
	}
}

		
