#include <cassert>
#include <iostream>


class Stat {
	int dim;
	int n;
	double* data;
	double xmin, xmax, xsum;
	void resize();
public:
	Stat();
	~Stat() {delete [] data;}
	void sample(const double);
	double min(){assert(n!=0); return xmin;}
	double max(){assert(n!=0); return xmax;}

	double mean(){assert (n!=0); return xsum/n;}

	double deviation();
};

Stat::Stat(): dim(10), n(0), data(new double[dim]), xsum(0){
	for (int i=0; i<dim; ++i) data[i]=0; 
}

void Stat::sample (const double xi) {
	if (n == 0) xmin=xmax=xi;
	if (n == dim) resize();
	data[n++]=xi; 
	if (xi<xmin) xmin=xi;
	if (xi>xmax) xmax=xi;
	xsum+=xi;
}

double Stat::deviation() {
	double s2=0, i2;
	for (int i=0; i<n; ++i) {
		i2=mean()-data[i];
		s2+=i2*i2;
	}
	return sqrt(s2);
}

void Stat::resize() {
	double* newdata=new double[dim=10*dim];
	for (int i=0; i<dim; ++i)
		newdata[i]=0;
	for (int i=0; i<n; ++i)
		newdata[i]=data[i];
	delete [] data;
	data=newdata;
}

int main()
{
	Stat s;

	for (int i=-100; i<=200; ++i) s.sample(i);

	cout<<s.mean()<<"    "<<s.deviation()<<endl;
}




	
	
	
