#include <iostream>
#include <cmath>

#define WIN32

#ifdef UNIX
#define _IEEE
#include <nan.h>
#else
#ifdef WIN32
#include <float.h>
#define NaN(X) _isnan(X)
#endif
#endif


// Umkreiskonstruktion

class Punkt {
public:
	double x, y;
	Punkt(double px=0, double py=0): x(px), y(py) {}
	double Abstand(const Punkt& p)
	{
		return sqrt((x-p.x)*(x-p.x)+(y-p.y)*(y-p.y));
	}
};

class Gerade {
protected:
	double m, n;
	// Anstiegsgleichung: y = m * x + n; falls Strecke nicht in y-Richtung
	// 		      x = n, m = Infinity sonst
public:
	Gerade (Punkt p1, Punkt p2)
	{
		m=(p2.y-p1.y)/(p2.x-p1.x);
		if (p1.x == p2.x) n=p1.x;
		else 
			n=(p1.y*p2.x-p1.x*p2.y)/(p2.x-p1.x);
	}
	Gerade (double pm, double pn):m(pm), n(pn) {}

	friend int parallel (const Gerade& s1, const Gerade& s2)
	{
		return s1.m==s2.m;
	}
	

	friend Punkt Schnittpunkt (const Gerade& s1, const Gerade& s2)
	{
		if (!parallel(s1, s2))
		{
			if (NaN(s1.m))
				return Punkt(s1.n, s1.n*s2.m+s2.n);
			if (NaN(s2.m))
				return Punkt(s2.n, s2.n*s1.m+s1.n);
			return Punkt((s2.n-s1.n)/(s1.m-s2.m), 
				(s1.m*s2.n-s1.n*s2.m)/(s1.m-s2.m));
		}
		else 
		{
			cerr << "kein Schnittpunkt\n";
			return Punkt(0,0);
		}
	}
};


class Strecke: Gerade {
	Punkt P1, P2;
public:
	Strecke (Punkt p1, Punkt p2):Gerade(p1,p2), P1(p1), P2(p2){}				
	Gerade Mittelsenkrechte ()
	{
		Punkt mitte ((P2.x + P1.x)/2, (P2.y + P1.y)/2);
	
		if (m==0) return Gerade(-1/m, mitte.x);
		else
			  return Gerade(-1/m, mitte.y+mitte.x/m);
	}
};


main()
{
	Punkt P[3];

	for (int i=0; i<3; ++i)
	{
		cout<<"Koordinaten des "<<i+1<<". Punktes:\n";
		cout<<"\tx = "; cin>>P[i].x;
		cout<<"\ty = "; cin>>P[i].y;
		cout<<"\n\n";
	}
	Strecke S1 (P[0], P[1]);
	Strecke S2 (P[1], P[2]);
	
	Gerade G1=S1.Mittelsenkrechte();
	Gerade G2=S2.Mittelsenkrechte();
	
	Punkt PX=Schnittpunkt(G1, G2);

	cout << "P1 --> PX: " << P[0].Abstand(PX) << endl;
	cout << "P2 --> PX: " << P[1].Abstand(PX) << endl;
	cout << "P3 --> PX: " << P[2].Abstand(PX) << endl;
}

	

