#include <iostream>
#include <algorithm>


class Polynom {
	int n; // n is order of polynom with n+1 coeffs
	double* c;
public:
	Polynom(int order, double coeff[]):n(order)
	{
		c=new double[n+1];
		for (int i=0; i<n+1; ++i) c[i]=coeff[i];
	}
	
	~Polynom(){delete[] c;}
	
	double eval(double x)
	{
		double res=0;
		for (int i=n; i>=0; --i)
			res=x*res+c[i];
		return res;
	}
	
	Polynom derive()
	{
		double* v = new double[n];
		for (int i=0; i<n; ++i) v[i]=c[i+1]*(i+1);
		Polynom p(n-1, v);
		delete v;
		return p;
	}
	
	friend ostream& operator<<(ostream& o, const Polynom& p)
	{
		if (p.c[0]) o<<p.c[0]<<"+";
		if (p.c[1]) {
			if (p.c[1]!=1) o<<p.c[1]<<"*";
			cout<<"x"<<"+";
		}
		for (int i=2; i<p.n; ++i)
			if (p.c[i]) {
				if (p.c[i]!=1) o<<p.c[i]<<"*";
				o<<"x^"<<i<<"+"; 
		}
		if (p.c[p.n]!=1) o<<p.c[p.n]<<"*";
		return o<<"x^"<<p.n;
	}
	
	friend Polynom operator+(const Polynom& p1, const Polynom& p2)
	{
		int m = max(p1.n, p2.n);
		double* v=new double [m+1];
		for (int i=0; i<=m; ++i) v[i]=p1.c[i]+p2.c[i];
		Polynom p(m, v);
		delete v;
		return p;
	}
};

int main()
{
	double a[]={1, 2, 3, 4, 5, 6};
	double b[]={6, 5, 4, 3, 2, 1};
	 
	Polynom p(5, a);
	Polynom q(5, b);
	
	
	cout<<p<<endl;
	cout<<q<<endl;
	cout<<p.derive()<<endl<<p.eval(1)<<endl;
	cout<<q.derive()<<endl<<q.eval(1)<<endl;
		
	cout<<p+q<<endl<<(p+q).derive()<<endl<<(p+q).eval(0.1)<<endl;

	
}

/*
1+2*x+3*x^2+4*x^3+5*x^4+6*x^5
6+5*x+4*x^2+3*x^3+2*x^4+x^5
2+6*x+12*x^2+20*x^3+30*x^4
21
5+8*x+9*x^2+8*x^3+5*x^4
21
7+7*x+7*x^2+7*x^3+7*x^4+7*x^5
7+14*x+21*x^2+28*x^3+35*x^4
7.77777
*/
	
	
