/*
nach einer Idee von Martin Schader und Stefan Kuhlins:
Programmieren in C++ - Einfuehrung in den Sprachstandard (4. Auflage)
Springer-Verlag 1996,
ISBN 3-540-61834-1.
*/

#include <iostream>
#include <cstdlib>

// namespace Chess { // not with g++

typedef enum {A,B,C,D,E,F,G,H} Col;
typedef int Row;

struct Pos {
	Row row;
	Col col;
	Pos(Col c, Row r):row(r), col(c){} 
};
	
class Figure {
	
public:
	Figure(Pos p):where(p){}
	virtual ~Figure(){}
	void move(Pos);	
	virtual bool canMoveTo (Pos) const = 0;

protected:
	bool onBoard(Pos) const;    // gueltiges Feld ?
  	bool wouldMove(Pos) const;  // wuerde sich was bewegen ?
	Pos where;
};

class Rook : virtual public Figure {	// Turm
public:
  Rook(Pos p) : Figure (p) { }
  bool canMoveTo (Pos) const;
};

class Bishop : virtual public Figure {	//Laeufer
public:
  Bishop(Pos p) : Figure(p) { }
  bool canMoveTo (Pos) const;
};

class Knight : virtual public Figure {	//Springer
public:
  Knight(Pos p) : Figure(p) { }
  bool canMoveTo (Pos) const;
};

// Die Dame zieht wie ein Turm oder Laeufer, aber sie ist kein
// Turm oder Laeufer.

class Queen : virtual public Figure, private Rook, private Bishop {
public:
  Queen(Pos);
  bool canMoveTo (Pos) const;
};

void Figure::move (Pos p) {
  if (canMoveTo(p)) where=p;
}

inline bool Figure::onBoard(Pos p) const {
  return A <= p.col && p.col <= H && 1 <= p.row && p.row <= 8;
}

inline bool Figure::wouldMove(Pos p) const {
  return p.col != where.col || p.row != where.row;
}

bool Rook::canMoveTo (Pos p) const {
  return onBoard(p) && wouldMove(p) && 
  		(where.col == p.col || where.row == p.row);
}

bool Bishop::canMoveTo(Pos p) const {
  return onBoard(p) && wouldMove(p) && 
  		(abs(where.col-p.col) == abs(where.row-p.row));
}

bool Queen::canMoveTo(Pos p) const {
  return Rook::canMoveTo(p) || Bishop::canMoveTo(p);
}

bool Knight::canMoveTo(Pos p) const {
  return onBoard(p) && wouldMove(p) && 
  		((abs(where.col-p.col)==1 && abs(where.row-p.row)==2) ||
		 (abs(where.col-p.col)==2 && abs(where.row-p.row)==1));
}

Queen::Queen(Pos p) : Figure(p), Rook(p), Bishop(p) { }

// }// namespace Chess not with g++

// using namespace Chess; not with g++

#define ON(C,R) (Pos(C,R))

int main() {
  Rook t ON (A, 1);
  Bishop l ON (C, 1);
  Queen d ON (D, 1);
  Knight s ON (B, 1);

  Figure* z = &d;
  // Turm* zt = &d;   // Fehler: eine Dame ist kein Turm (private abgeleitet)

  cout << "Dd1-d8 " << z->canMoveTo(Pos(D, 8)) << endl;   // Turmzug der Dame
  z->move(Pos(D, 8));

  cout << "Dd8-f6 " << z->canMoveTo(Pos(F, 6)) << endl;   // Laeuferzug der Dame
  z->move(Pos(F, 6));

  cout << "Dd7 " << d.canMoveTo(Pos(D, 7)) << endl;       // Df6-d7 geht nicht
  d.move(Pos(D, 7));

  cout << "Df6 " << d.canMoveTo(Pos(F, 6)) << endl;       // Df6-f6 geht auch nicht
  d.move(Pos(F, 6));

  cout << "Ta1-a8 " << t.canMoveTo(Pos(A, 8)) << endl;    // Turmzug
  t.move(Pos(A, 8));

  cout << "Lc1-g5 " << l.canMoveTo(Pos(G, 5)) << endl;    // Laeuferzug
  l.move(Pos(G, 5));

  cout << "Sb1-c3 " << s.canMoveTo(Pos(C, 3)) << endl;    // Springerzug
  l.move(Pos(C, 3));

}

	
